﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Input;

namespace Common.MVVM.Messages
{
    //allows other objects to invoke delgates
    public  class InvokeCommand : ICommand
    {
        private readonly Action<object> _execute;
        private readonly Predicate<object> _canExecute;

        #region CTORS

        public InvokeCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        public InvokeCommand(Action<object> execute, Predicate<object> canExecute)
        {
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            if (parameter == null)
                throw new ArgumentNullException("execute");


            _execute(parameter);
        }



        #endregion


    }
}
