﻿using System.Collections.Generic;
using System.Linq;
using Unity;

namespace Common.Unity
{
    public static class UnityService
    {
        private static UnityContainer _container;

        /// <summary>
        /// Holds the registration of interfaces and their concrete classes.
        /// </summary>
        public static UnityContainer GetContainer
        {
            get
            {
                if (_container == null)
                {
                    _container = new UnityContainer();
                }

                return _container;
            }
        }

        /// <summary>
        /// Unity resolves the type T by creating T's registered concrete class from UnityContainer.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns>The registered concrete class.</returns>
        public static T Resolve<T>()
        {
            try
            {
                if (_container == null) return default(T);

                return _container.Resolve<T>();
            }
            catch (System.Exception)
            {
#if DEBUG
                // Stops the execution if T has not been registered with Unity.
                System.Diagnostics.Debugger.Break();
                throw;
#else
                throw;
#endif
            }
        }


        /// <summary>
        /// Unity resolves the type T by creating a list of T's registered concrete class from UnityContainer.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns>A list of the registered concrete class.</returns>
        public static IList<T> ResolveList<T>()
        {
            try
            {
                if (_container == null) return null;

                IList<T> list = _container.ResolveAll<T>().ToList();

                return list;
            }
            catch (System.Exception)
            {
#if DEBUG
                // Stops the execution if T has not been registered with Unity.
                System.Diagnostics.Debugger.Break();
                throw;
#else
                throw;
#endif
            }
        }


    }
}
