﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using Common.MVVM.Messages;
using Common.Unity;
using MvvmBinding.Models.Countries;
using MvvmBinding.Models.Countries.Interfaces;
using MvvmBinding.Models.Customers;
using MvvmBinding.Models.Customers.Interfaces;
using MvvmBinding.ViewModels.Customers;
using MvvmBinding.ViewModels.Customers.Interfaces;
using MvvmBinding.ViewModels.Dashboard;
using MvvmBinding.ViewModels.Dashboard.Interfaces;
using Unity;

namespace MvvmBinding
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        private static UnityContainer _container;

        public App()
        {
            if (Dispatcher == null) throw new InvalidOperationException("App.xaml.cs - Dispatcher is null.");

            Dispatcher.UnhandledException += OnDispatcherUnhandledException;

            ConfigureUnity();

            InitializeEventProvider();

            IShell shellViewModel = _container.Resolve<IShell>();
        }

        /// <summary>
        /// Catch unhandled exceptions thrown on the main UI thread and allow 
        /// option for user to continue program. 
        /// The OnDispatcherUnhandledException method below for AppDomain.UnhandledException will handle all other exceptions thrown by any thread.
        /// </summary>
        void AppUI_DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            if (e.Exception == null)
            {
                Application.Current.Shutdown();
                return;
            }
            string errorMessage = string.Format("An application error occurred. If this error occurs again there seems to be a serious bug in the application, and you better close it.\n\nError:{0}\n\nDo you want to continue?\n(if you click Yes you will continue with your work, if you click No the application will close)", e.Exception.Message);
            //insert code to log exception here
            if (MessageBox.Show(errorMessage, "Application User Interface Error", MessageBoxButton.YesNoCancel, MessageBoxImage.Error) == MessageBoxResult.No)
            {
                if (MessageBox.Show("WARNING: The application will close. Any changes will not be saved!\nDo you really want to close it?", "Close the application!", MessageBoxButton.YesNoCancel, MessageBoxImage.Warning) == MessageBoxResult.Yes)
                {
                    Application.Current.Shutdown();
                }
            }
            e.Handled = true;
        }

        /// <summary>
        /// Catch unhandled exceptions not thrown by the main UI thread.
        /// The above AppUI_DispatcherUnhandledException method for DispatcherUnhandledException will only handle exceptions thrown by the main UI thread. 
        /// Unhandled exceptions caught by this method typically terminate the runtime.
        /// </summary>
        void OnDispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            string errorMessage = string.Format("An application error occurred. If this error occurs again there seems to be a serious bug in the application, and you better close it.\n\nError:{0}\n\nDo you want to continue?\n(if you click Yes you will continue with your work, if you click No the application will close)", e.Exception.Message);
            //insert code to log exception here
            if (MessageBox.Show(errorMessage, "Application UnhandledException Error", MessageBoxButton.YesNoCancel, MessageBoxImage.Error) == MessageBoxResult.No)
            {
                if (MessageBox.Show("WARNING: The application will close. Any changes will not be saved!\nDo you really want to close it?", "Close the application!", MessageBoxButton.YesNoCancel, MessageBoxImage.Warning) == MessageBoxResult.Yes)
                {
                    Application.Current.Shutdown();
                }
            }
            e.Handled = true;
        }


        //public IConfiguration Configuration { get; set; }

        public void App_OnStartup(object sender, StartupEventArgs e)
        {
            
        }


        /// <summary>
        /// Configures Unity by registering the concrete classes and their interfaces.
        /// </summary>
        private void ConfigureUnity()
        {
            _container = UnityService.GetContainer;

            _container.RegisterType<IEventAggregator, EventAggregator>();
            _container.RegisterType<IShell, ShellViewModel>();


            _container.RegisterType<ICustomerEditViewModel, CustomerEditViewModel>();
            _container.RegisterType<ICustomerListViewModel, CustomerListViewModel>();

            _container.RegisterType<IDashboardViewModel, DashboardViewModel>();
            
            _container.RegisterType<ICountry, Country>();

            _container.RegisterType<ICustomer, Customer>();
        }


        private void InitializeEventProvider()
        {
            IEventAggregator eventAggregator = _container.Resolve<IEventAggregator>();

            if (eventAggregator == null) throw new InvalidOperationException("App - eventAggregator is null.");

            EventProvider.StartEventAggregator(eventAggregator);
        }


    }
}
