﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.MVVM.Models;
using Common.Unity;
using MvvmBinding.Helpers;
using MvvmBinding.Models.Countries.Interfaces;
using MvvmBinding.Models.Customers.Interfaces;

namespace MvvmBinding.Models.Customers
{
    public class CustomerEditModel : BusinessBase<CustomerEditModel>
    {
        private string _customerName;
        private int _countryId;
        private DateTime _customerDateJoined;

        public IList<ICountry> Countries { get; set; }

        public string CustomerName
        {
            get { return _customerName; }
            set { SetField(ref _customerName, value, nameof(CustomerName)); }
        }

        public int CountryId
        {
            get { return _countryId; }
            set { SetField(ref _countryId, value, nameof(CountryId)); }
        }


        public DateTime CustomerDateJoined
        {
            get { return _customerDateJoined; }
            set { SetField(ref _customerDateJoined, value, nameof(CustomerDateJoined)); }
        }



        


        /// <summary>
        /// Constructor.
        /// </summary>
        public CustomerEditModel()
        {
            Countries = UnityService.ResolveList<ICountry>();
        }

        public void GetCountries()
        {
            Countries = CountryMockData.GetCountries();

            ModelHasChanged(nameof(Countries));
        }

        public void PopulateCustomer(ICustomer customer)
        {
            if (customer == null) throw new ArgumentNullException(nameof(customer));

            CustomerName = customer.CustomerName;
            CustomerDateJoined = customer.CustomerDateJoined;

            CountryId = customer.CountryId;
            ModelHasChanged(nameof(CountryId));
        }


        protected override bool PropertyHasChangedCheckModelIsDirty()
        {
            return true;
        }


    }
}
