﻿using System.Windows;
using System.Windows.Controls.Primitives;
using Infragistics.Controls.Interactions;

namespace MvvmBinding.Views.Customers
{
    /// <summary>
    /// Interaction logic for CustomerEditView.xaml
    /// </summary>
    public partial class CustomerEditView : XamDialogWindow
    {
        Popup _hostPopup;
        Window _parentWindow;

        public CustomerEditView()
        {
            InitializeComponent();
            WindowStateChanged += Dialog_WindowStateChanged;

            _hostPopup = new Popup();
            _hostPopup.Child = this;
        }

        void Dialog_WindowStateChanged(object sender, WindowStateChangedEventArgs e)
        {
            if (e.NewWindowState == Infragistics.Controls.Interactions.WindowState.Hidden)
            {

                if (_parentWindow != null)
                {
                    //unhook our event handlers to prevent memory leaks
                    _parentWindow.LocationChanged -= ParentWindow_LocationChanged;
                    _parentWindow.SizeChanged -= ParentWindow_SizeChanged;
                }

                if (_hostPopup != null)
                {
                    _hostPopup.IsOpen = false;
                }

            }
        }

        public void ShowDialog()
        {
            base.Show();

            _hostPopup.AllowsTransparency = true;

            foreach (Window window in App.Current.Windows)
            {
                if (window.IsActive) //I want to show the popup over the current active window
                {
                    _parentWindow = window;

                    _hostPopup.PlacementTarget = _parentWindow;
                    _hostPopup.Placement = PlacementMode.Relative;

                    _hostPopup.HorizontalOffset = -7.5; //account for window padding
                    _hostPopup.VerticalOffset = -30.5; //account for window title bar

                    CalculatePopupSize();

                    _parentWindow.LocationChanged += ParentWindow_LocationChanged;
                    _parentWindow.SizeChanged += ParentWindow_SizeChanged;

                    break;
                }
            }

            _hostPopup.IsOpen = true;

        }


        void CalculatePopupSize()
        {
            _hostPopup.Width = _parentWindow.Width;
            _hostPopup.Height = _parentWindow.Height;
        }

        void ParentWindow_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            CalculatePopupSize();
        }

        void ParentWindow_LocationChanged(object sender, System.EventArgs e)
        {
            //this forces the popup to re-evaluate it's positioning
            _hostPopup.Placement = PlacementMode.Custom;
            _hostPopup.Placement = PlacementMode.Relative;
        }

    }
}
